private_key = "-"
private_text = "PRIVATE"
public_key = "+"
public_text = "PUBLIC"

format_list = [
    "https://{region}.amazonaws.com/{word}{sep}{env}",
    "https://{region}.amazonaws.com/{env}{sep}{word}",
    # "https://{word}{sep}{env}.{region}.amazonaws.com",
    # "https://{env}{sep}{word}.{region}.amazonaws.com",
]

useragent_list = [
    "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36",
    "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36",
    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_12_5) AppleWebKit/603.2.4 (KHTML, like Gecko) Version/10.1.1 Safari/603.2.4",
]



import re

import setuptools, os


def open_local(paths, mode="r", encoding="utf8"):
    path = os.path.join(os.path.abspath(os.path.dirname(__file__)), *paths)
    return open(path, mode=mode, encoding=encoding)


with open_local(["s3recon", "__init__.py"]) as f:
    version = re.search(r"__version__ = [\"'](\d+\.\d+\.\d+)[\"']", f.read()).group(1)

with open_local(["README.md"]) as f:
    long_description = f.read()

install_requires = ["mergedeep>=1.0"]

setuptools.setup(
    name="s3recon",
    version=version,
    author="Travis Clarke",
    author_email="travis.m.clarke@gmail.com",
    description="Amazon S3 bucket finder.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/clarketm/s3recon",
    packages=setuptools.find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    entry_points={"console_scripts": ["s3recon=s3recon.s3recon:cli"]},
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)



#!/usr/bin/env python3
from asyncio import get_event_loop, gather
from collections import defaultdict
from datetime import datetime
from json import dumps
from logging import getLogger, basicConfig, INFO
from os import environ
from pathlib import Path
from random import choice
from sys import stdout, path
from warnings import filterwarnings

import requests
from mergedeep import merge
from requests import RequestException
from urllib3.exceptions import InsecureRequestWarning
from yaml import load

if not __package__:
    path.insert(0, str(Path(Path(__file__).parent.parent.parent)))


from s3recon import __version__
from s3recon.constants import (
    useragent_list,
    format_list,
    public_key,
    private_key,
    public_text,
    private_text,
)

filterwarnings("ignore", category=InsecureRequestWarning)

logger = getLogger(__name__)

# TODO: opt to change log-level
basicConfig(format="%(message)s", level=INFO)


def bucket_exists(url, timeout):
    exists = False
    public = False

    try:
        res = requests.head(
            url,
            headers={"User-Agent": choice(useragent_list)},
            verify=False,
            timeout=timeout,
        )
        # TODO: handle redirects
        status_code = res.status_code
        exists = status_code != 404
        public = status_code == 200
    except RequestException:
        pass

    return exists, public


def find_bucket(url, timeout):
    exists, public = bucket_exists(url, timeout)

    if exists:
        access = public_key if public else private_key
        access_word = public_text if public else private_text
        logger.info(f"{access} {access_word} {url}")
        return access, url

    return None


def collect_results(r):
    d = defaultdict(list)
    for access, url in filter(bool, r):
        d[access].append(url)

    return d.get("-", []), d.get("+", [])


def read_config():
    config = {}

    config_hierarchy = [
        Path(Path(__file__).parent, "s3recon.yml"),  # default
        Path(Path.home(), "s3recon.yaml"),
        Path(Path.home(), "s3recon.yml"),
        Path(Path.cwd(), "s3recon.yaml"),
        Path(Path.cwd(), "s3recon.yml"),
        Path(environ.get("S3RECON_CONFIG") or ""),
    ]

    for c in config_hierarchy:
        try:
            c = load(open(c, "r")) or {}
            merge(config, c)
        except (IOError, TypeError):
            pass

    return config


def main(words, timeout, output, only_public):
    start = datetime.now()
    loop = get_event_loop()

    config = read_config()
    regions = config.get("regions") or [""]
    separators = config.get("separators") or [""]
    environments = config.get("environments") or [""]

    url_list = {
        f.format(
            region=f"s3.{region}" if region else "s3",
            word=word,
            sep=sep if env else "",
            env=env,
        )
        for f in format_list
        for region in regions
        for word in words
        for sep in separators
        for env in environments
    }

    tasks = gather(
        *[loop.run_in_executor(None, find_bucket, url, timeout) for url in url_list]
    )
    r = loop.run_until_complete(tasks)

    private, public = collect_results(r)
    stop = datetime.now()

    logger.info(f"Complete after: {stop - start}")
    logger.info(f"Output written to: {output.name}")

    results = {
        "private": {"total": len(private), "hits": private},
        "public": {"total": len(public), "hits": public},
    }

    if only_public:
        del results["private"]

    output.write(dumps(results, indent=4))


def cli():
    import argparse

    parser = argparse.ArgumentParser(
        description=__doc__,
        formatter_class=lambda prog: argparse.HelpFormatter(
            prog, max_help_position=35, width=100
        ),
    )
    parser.add_argument(
        "-o",
        "--output",
        type=argparse.FileType("w"),
        metavar="file",
        default=stdout,
        help="write output to <file> (default: stdout)",
    )
    parser.add_argument(
        "-p",
        "--public",
        action="store_true",
        help="only include 'public' buckets in the output",
    )
    parser.add_argument(
        "-t",
        "--timeout",
        type=int,
        metavar="seconds",
        default=30,
        help="http request timeout in <seconds> (default: 30)",
    )
    parser.add_argument(
        "-v", "--version", action="version", version=f"%(prog)s {__version__}"
    )
    # parser.add_argument("words", nargs="?", type=argparse.FileType("r"), default=stdin, help="list of words to permute")
    parser.add_argument(
        "word_list",
        nargs="+",
        type=argparse.FileType("r"),
        help="read words from one or more <word-list> files",
    )
    args = parser.parse_args()

    output = args.output
    timeout = args.timeout
    public = args.public
    words = {l.strip() for f in args.word_list for l in f}

    main(words=words, timeout=timeout, output=output, only_public=public)


if __name__ == "__main__":
    cli()
